/* Text clustering method, SKWIC algorithm implementation
 * Main model class to store input data
 * Authors: Roman Milos, Vladimir Bartik, May 2011
 */
package dip.model;

import java.io.File;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlElementWrapper;
import javax.xml.bind.annotation.XmlRootElement;

/**
 * Main model class to store input data
 * @author Roman
 */
@XmlRootElement
@XmlAccessorType(XmlAccessType.NONE)
public class Data {

    /*
     * Enumeration consisting of possible weighting methods. 
     */
    public enum VahovaciMetody {

        TF_IDF("TF-IDF"),
        TF("TF"),
        IDF("IDF");
        private final String label;

        private VahovaciMetody(String label) {
            this.label = label;
        }

        @Override
        public String toString() {
            return label;
        }
    }
    //optimal count of terms for a hash map
    private final int VELIKOST_HASH_MAPY = 10000;
    //Attributes used by the application:
    //Data singleton creation:
    private static Data instance = new Data();
    //Weighting method
    @XmlElement
    private VahovaciMetody vahovani = VahovaciMetody.IDF;
    //Configuration of pre-processing and algorithm:
    @XmlElement
    private int maximalniPoceTermu = 200000;
    private boolean aplikovatStopList = false;
    @XmlElement
    private boolean aplikovatStemming = true;
    @XmlElement
    private boolean mazatCisla = true;
    @XmlElement
    private boolean mazatTermyJednohoDokumentu = true;
    @XmlElement
    private int pocetTrid = 1;
    @XmlElement
    private double konstanta = 1;
    @XmlElement(name = "data")
    private File dataDir;
    @XmlElement(name = "stopSlova")
    private File stopListFile;
    @XmlElement
    private File centraShluku;
    //Hash mapy:
    private HashMap<String, Boolean> stopSlova;
    private HashMap<String, Integer> kategorie;
    private HashMap<String, SlovoSuma> slovaDokumenty;
    @XmlElementWrapper(name = "kategorieKolekce")
    @XmlElement(name = "kategorie")
    private Collection<String> kategorieList;
    @XmlElementWrapper(name = "termy")
    @XmlElement(name = "term")
    private Collection<SlovoSuma> slovaDokumentyKolekce;
    @XmlElementWrapper(name = "dokumenty")
    @XmlElement(name = "dokument")
    private List<Dokument> dokumenty;

    /**
     * Constructor - initialization of structures.
     */
    private Data() {

        clean();

    }

    /**
     * Preparation for export - conversion of hash structures into collections. 
     * Conversion is made for all documents. 
     */
    public void pripravNaExport() {

        slovaDokumentyKolekce = slovaDokumenty.values();
        for (Dokument d : dokumenty) {
            d.pripravNaExport();
        }
        for (String kat : kategorie.keySet()) {
            kategorieList.add(kat);
        }
        if (!isAplikovatStopList()) {
            stopListFile = null;
        }
    }

    /**
     * Correcting data after import. Converts collections into hash structures 
     * and corrects documents. 
     * @throws Exception
     */
    public void opravPoImportu() throws Exception {
        slovaDokumenty.clear();

        if (slovaDokumentyKolekce != null) {

            if (slovaDokumentyKolekce.size() > 0) {

                for (SlovoSuma s : slovaDokumentyKolekce) {
                    slovaDokumenty.put(s.getJmeno(), s);
                }
            }

        }
        if (kategorieList != null) {

            if (kategorieList.size() > 0) {
                int i = 0;
                for (String kat : kategorieList) {
                    kategorie.put(kat, i);
                    i++;
                }
            }

        }

        for (Dokument d : dokumenty) {

            d.opravPoImportu();
            for (Slovo s : d.getSlova().values()) {
                if (!slovaDokumenty.containsKey(s.getJmeno())) {
                    throw new Exception("Incorrect data");
                }
            }
        }


    }


    /**
     * Gets the value of constant. 
     * @return constant value
     */    
    public double getKonstanta() {
        return konstanta;
    }
    
    /**
     * Setting of the constant. 
     * @param konstanta value of the constant
     */
    public void setKonstanta(double konstanta) {
        this.konstanta = konstanta;
    }
    
    /**
     * Gets the value of maximum count of terms. 
     * @return maximum count of terms
     */   
    public int getMaximalneTermu() {
        return maximalniPoceTermu;
    }
    
    /**
     * Setting the maximum count of terms.
     * @param maximalniPocetTermu maximum count of terms
     */
    public void setMaximaleTermu(int maximalniPocetTermu) {
        this.maximalniPoceTermu = maximalniPocetTermu;
    }
    
    /**
     * Gets the list of all categories. 
     * @return list of categories
     */   
    public Collection<String> getKategorieList() {
        return kategorieList;
    }
    
    /**
     * Setting the list of all categories. 
     * @param kategorieList list of categories
     */
    public void setKategorieList(Collection<String> kategorieList) {
        this.kategorieList = kategorieList;
    }
    
    /**
     * Gets the hash map with a list of categories. 
     * @return a hash map of categories
     */   
    public HashMap<String, Integer> getKategorie() {
        return kategorie;
    }
    
    /**
     * Setting the categories from an input hash map. 
     * @param kategorie an input hash map
     */
    public void setKategorie(HashMap<String, Integer> kategorie) {
        this.kategorie = kategorie;
    }

    /**
     * Gets the centers of all clusters in a form of file. 
     * @return a file with cluster centers
     */   
    public File getCentraShluku() {
        return centraShluku;
    }
    
    /**
     * Setting the cluster centers from a file.
     * @param centraShluku file with cluster centers
     */
    public void setCentraShluku(File centraShluku) {
        this.centraShluku = centraShluku;
    }


    /**
     * Gets the value of class count. 
     * @return count of classes
     */   
    public int getPocetTrid() {
        return pocetTrid;
    }
    
    /**
     * Setting the value of the count of classes. 
     * @param pocetTrid count of classes
     */
    public void setPocetTrid(int pocetTrid) {
        this.pocetTrid = pocetTrid;
    }

    /**
     * Gets the collection of all documents. 
     * @return collection of documents
     */   
    public Collection<SlovoSuma> getSlovaDokumentyKolekce() {
        return slovaDokumentyKolekce;
    }

    /**
     * Setting the collection of words in documents. 
     * @param slovaDokumentyKolekce collection of words in documents
     */    
    public void setSlovaDokumentyKolekce(Collection<SlovoSuma> slovaDokumentyKolekce) {
        this.slovaDokumentyKolekce = slovaDokumentyKolekce;
    }

    /**
     * Gets the value of option, if the application will remove terms 
     * contained in only one document.
     * @return boolean value
     */   
    public boolean isMazatTermyJednohoDokumentu() {
        return mazatTermyJednohoDokumentu;
    }
    
    /**
     * Setting the option, if the application will remove terms contained in only
     * one document. 
     * @param mazatTermyJednohoDokumentu boolean value
     */  
    public void setMazatTermyJednohoDokumentu(boolean mazatTermyJednohoDokumentu) {
        this.mazatTermyJednohoDokumentu = mazatTermyJednohoDokumentu;
        //   System.err.println(instance);
    }
    
    /**
     * Gets the actual weighting method to be used. 
     * @return a weighting method
     */ 
    public VahovaciMetody getVahovani() {
        return vahovani;
    }

    /**
     * Setting the actual weighting method to be used. 
     * @param vahovani a weighting method
     */     
    public void setVahovani(VahovaciMetody vahovani) {
        this.vahovani = vahovani;
    }

    /**
     * Gets the hash map of words assigned to a document. 
     * @return a hash map of words in a document
     */     
    public HashMap<String, SlovoSuma> getSlovaDokumenty() {
        return slovaDokumenty;
    }

    /**
     * Setting the words assigned for a document. 
     * @param slovaDokumenty a hash map of words in a document
     */ 
    public void setSlovaDokumenty(HashMap<String, SlovoSuma> slovaDokumenty) {
        this.slovaDokumenty = slovaDokumenty;
    }

    /**
     * Gets the value of option, if the application will remove digits from
     * document representation.
     * @return boolean value
     */  
    public boolean isMazatCisla() {
        return mazatCisla;
    }

    /**
     * Setting the option, if the application will remove digits from document
     * representation. 
     * @param mazatCisla boolean value
     */  
    public void setMazatCisla(boolean mazatCisla) {
        this.mazatCisla = mazatCisla;
    }
    
    /**
     * Gets the value of option, if the application will apply a stemming 
     * algorithm. 
     * @return boolean value
     */  
    public boolean isAplikovatStemming() {
        return aplikovatStemming;
    }
    
    /**
     * Setting the option, if the application will apply a stemming algorithm. 
     * @param aplikovatStemming boolean value
     */  
    public void setAplikovatStemming(boolean aplikovatStemming) {
        this.aplikovatStemming = aplikovatStemming;
    }
    
    /**
     * Gets the value of option, if the application will use a stoplist.
     * @return boolean value
     */  
    public boolean isAplikovatStopList() {
        return aplikovatStopList;
    }
    
    /**
     * Setting the option, if the application will use a stoplist. 
     * @param aplikovatStopList boolean value
     */  
    public void setAplikovatStopList(boolean aplikovatStopList) {
        this.aplikovatStopList = aplikovatStopList;
    }
    
    /**
     * Gets the hash map containing the stop words.
     * @return a stoplist
     */  
    public HashMap<String, Boolean> getStopSlova() {
        return stopSlova;
    }

    /**
     * Setting the contents of stoplist. 
     * @param stopSlova a hash map of stop words
     */  
    public void setStopSlova(HashMap<String, Boolean> stopSlova) {
        this.stopSlova = stopSlova;
    }

    /**
     * Gets the instance of this class.
     * @return an instance of Data class
     */  
    public static Data getInstance() {
        return Data.instance;
    }
    
    /**
     * Gets the list of documents.
     * @return list of documents
     */  
    public List<Dokument> getDokumenty() {
        return dokumenty;
    }
    
    /**
     * Setting the list of documents. 
     * @param dokumenty a list of documents
     */  
    public void setDokumenty(List<Dokument> dokumenty) {
        this.dokumenty = dokumenty;
    }
    
    /**
     * Gets the folder, in which input data are stored.
     * @return a folder with data
     */  
    public File getDataFile() {
        return dataDir;
    }
    
    /**
     * Setting of a folder, in which input data are contained.
     * @param dataDir a folder
     * @throws Exception
     */
    public void setDataFile(File dataDir) throws Exception {
        if (dataDir != null) {
            testDataDir(dataDir);
        }
        this.dataDir = dataDir;
    }

    /**
     * Clean all classes. 
     */
    public void clean() {
        dokumenty = new ArrayList<Dokument>();
        stopSlova = new HashMap<String, Boolean>();
        kategorie = new HashMap<String, Integer>();
        kategorieList = new ArrayList<String>();
        slovaDokumenty = new HashMap<String, SlovoSuma>(VELIKOST_HASH_MAPY);
        slovaDokumentyKolekce = new ArrayList<SlovoSuma>();


    }

    /**
     * Setting the file folder and file validity test. 
     * @param jmeno file name
     * @throws Exception
     */
    public void setDataDirString(String jmeno) throws Exception {
        File dataDirNova;

        try {
            dataDirNova = new java.io.File(jmeno);

        } catch (Exception e) {
            throw new Exception("Unable to access folder with data");
        }
        testDataDir(dataDirNova);
        this.dataDir = dataDirNova.getAbsoluteFile();
    }

    /**
     * Tests the correctness of input data file. 
     * @param jmeno input data file
     * @throws Exception
     */
    public void testDataDir(File jmeno) throws Exception {

        if (!jmeno.exists()) {
            throw new Exception("File(s) with data does not exist");
        }
        if (!jmeno.isDirectory() && !jmeno.isFile()) {
            throw new Exception("Data file is not a folder or file");
        }
        if (!jmeno.canRead()) {
            throw new Exception("Unable to read data");
        }

    }

    /**
     * Returns a stop words file.
     * @return file with a stoplist.
     */
    public File getStopListFile() {
        return stopListFile;
    }

    /**
     * Sets a stop words file.
     * @param stopListFile file with a stoplist.
     */
    public void setStopListFile(File stopListFile) throws Exception {
        if (stopListFile != null) {
            testStopListFile(stopListFile);
        }
        this.stopListFile = stopListFile;
    }

    /**
     * Sets a stop words file and runs the test of it.
     * @param stopListFile file with a stoplist.
     */
    public void setTestStopListFile(File stopListFile) throws Exception {
        if (stopListFile != null) {
            testStopListFile(stopListFile);
        }
        this.stopListFile = stopListFile;
    }

    /**
     * Sets a stoplist file from the file name given as a parameter.
     * @param jmeno file name of stoplist file
     * @throws Exception
     */
    public void setStopListString(String jmeno) throws Exception {
        File stopListFileNova;
        try {
            stopListFileNova = new java.io.File(jmeno);
        } catch (Exception e) {
            throw new Exception("Unable to access file with stop words");
        }
        testStopListFile(stopListFileNova);
        this.stopListFile = stopListFileNova;
    }

    /**
     * test of a stoplist file, eventually an exception is created.
     * @param jmeno file name of a stoplist file
     * @throws Exception
     */
    public void testStopListFile(File jmeno) throws Exception {

        if (!jmeno.exists()) {
            throw new Exception("Stop words file does not exist");
        }
        if (!jmeno.isFile()) {
            throw new Exception("Stop words file is not a file");
        }
        if (!jmeno.canRead()) {
            throw new Exception("Unable to read stop words file");
        }

    }

    /**
     * Clearing of all collections. 
     */
    public void reInit() {
        stopSlova.clear();
        dokumenty.clear();
        slovaDokumenty.clear();
        kategorie.clear();
    }
}
