/* Text clustering method, SKWIC algorithm implementation
 * Model class representing a cluster
 * Authors: Roman Milos, Vladimir Bartik, May 2011
 */
package dip.model.algoritmus;

import dip.model.Data;
import dip.model.Dokument;
import dip.model.SlovoSuma;
import dip.model.Slovo;
import java.util.ArrayList;

import java.util.Collections;
import java.util.List;

/**
 * Model class representing a cluster.
 * @author Roman
 */
public class Shluk {

    private Data data;
    //documents in cluster
    private List<DokumentShluku> dokumentyShluku;
    //centers
    private double centrum[];
    //weights
    private double vahy[];

    public double getVzdalenostPrazdnehoDokumentu() {
        return vzdalenostPrazdnehoDokumentu;
    }

    public void setVzdalenostPrazdnehoDokumentu(double vzdalenostPrazdnehoDokumentu) {
        this.vzdalenostPrazdnehoDokumentu = vzdalenostPrazdnehoDokumentu;
    }
    //initialization document
    private String stred;
    private double delta = 1;
    private double vzdalenostPrazdnehoDokumentu;
    
    /**
     * Gets the representative center of the initialization cluster. 
     * @return a value of center in a form of string
     */
    public String getStred() {
        return stred;
    }
    
    /**
     * Sets the representative center of the cluster. 
     * @param stred a value of center in a form of string
     */
    public void setStred(String stred) {
        this.stred = stred;
    }
    
    /**
     * Gets the array of document centers. 
     * @return array of document centers
     */
    public double[] getCentrum() {
        return centrum;
    }

    /**
     * Sets the array of document centers. 
     * @param centrum array of document centers
     */
    public void setCentrum(double[] centrum) {
        this.centrum = centrum;
    }
    
    /**
     * Gets the delta value of the cluster. 
     * @return double value of delta
     */
    public double getDelta() {
        return delta;
    }
    
    /**
     * Sets the delta value of the cluster. 
     * @param delta double value of delta
     */
    public void setDelta(double delta) {
        this.delta = delta;
    }
    
    /**
     * Gets the list of documents assigned to this cluster. 
     * @return list of documents
     */
    public List<DokumentShluku> getDokumentyShluku() {
        return dokumentyShluku;
    }
    
    /**
     * Sets the list of documents assigned to this cluster. 
     * @param dokumentyShluku list of documents
     */
    public void setDokumentyShluku(List<DokumentShluku> dokumentyShluku) {
        this.dokumentyShluku = dokumentyShluku;
    }
    
    /**
     * Sets the vector of weights assigned to the cluster. 
     * @return weight vector in a form of array
     */
    public double[] getVahy() {
        return vahy;
    }
    
    /**
     * Sets the vector of weights assigned to the cluster. 
     * @param vahy weight vector in a form of array
     */
    public void setVahy(double[] vahy) {
        this.vahy = vahy;
    }

    /**
     * Constructor, which creates a new cluster with a given center. Makes 
     * necessary initialization.
     * @param inicializacniStred a document representing the initialization center
     */
    public Shluk(Dokument inicializacniStred) {

        data = Data.getInstance();
        int velikost = data.getSlovaDokumenty().size();
        centrum = new double[velikost];
        vahy = new double[velikost];
        int i = 0;
        double inicializacniVaha = 1.0 / velikost;
        double sumaNorm = 0;
        //clears centres and weights
        for (i = 0; i < velikost; i++) {
            centrum[i] = 0;
            vahy[i] = inicializacniVaha;
        }
        //change weight of centre
        for (Slovo s : inicializacniStred.getSlova().values()) {
            i = data.getSlovaDokumenty().get(s.getJmeno()).getId();
            centrum[i] = s.getVaha();
            sumaNorm += s.getVaha();
        }
        //normalize centers
        for (Slovo s : inicializacniStred.getSlova().values()) {
            i = data.getSlovaDokumenty().get(s.getJmeno()).getId();
            centrum[i] /= sumaNorm;
        }
        //distance of empty document
        aktualizujVzdalenostPrazdnehoDokumentu();
        dokumentyShluku = new ArrayList<DokumentShluku>();
    }

    /**
     * Cleans the old values and counts all cluster centers. 
     */
    public void obnovStredy() {
        int i;
        int pocetDokumentu = dokumentyShluku.size();
        for (i = 0; i < centrum.length; i++) {
            centrum[i] = 0;
        }
        for (DokumentShluku dokumentShluku : dokumentyShluku) {
            for (Slovo s : dokumentShluku.getDokument().getSlova().values()) {
                i = data.getSlovaDokumenty().get(s.getJmeno()).getId();
                centrum[i] += s.getVaha();
            }

        }
        double sumaNorm = 0;
        for (i = 0; i < centrum.length; i++) {
            if (vahy[i] > 0) {
                centrum[i] /= pocetDokumentu;
                sumaNorm += centrum[i];
            } else {
                centrum[i] = 0;
            }

        }
        for (i = 0; i < centrum.length; i++) {
            centrum[i] /= sumaNorm;
        }
        aktualizujVzdalenostPrazdnehoDokumentu();

    }

    /**
     * Counts new values of document weights and normalizes them. 
     * @return count of negative weights.
     */
    public int spoctiVahy() {
        double vahaTermu = 1.0 / vahy.length;
        int i;
        double vahaNulovych = vahaTermu * dokumentyShluku.size();
        for (i = 0; i < vahy.length; i++) {
            vahy[i] = vahaNulovych;
        }
        for (DokumentShluku dokumentShluku : dokumentyShluku) {
            for (Slovo s : dokumentShluku.getDokument().getSlova().values()) {
                i = data.getSlovaDokumenty().get(s.getJmeno()).getId();
                vahy[i] += dokumentShluku.getVzdalenostTermu(s.getId()) - vahaTermu;
            }
        }
        //get delta and modify value
        double deltaProSumu = 0.5 / (delta);

        double vaha = 0;
        double minimum = 100;
        
        for (i = 0; i < vahy.length; i++) {


            vaha = vahy[i];
            vaha = (vaha * vahaTermu * dokumentyShluku.size()) - vaha;

            vahy[i] = vahaTermu +
                    vaha * deltaProSumu;
            //centrum[i]=vahaTermu;
            // centrum[i]=vaha;
            if (minimum > vahy[i]) {
                minimum = vahy[i];
            }
        }
        double sumaNorm = 0;
        int cnt = 0;

        //sum for normalization
        for (i = 0; i < vahy.length; i++) {
            if (vahy[i] < 0) {
                cnt++;
                vahy[i] -= minimum;
            }
            sumaNorm += vahy[i];

        }
        if (cnt > 0) {
            //System.err.println(stred + ": " + sumaNorm + " " + cnt);
        }
        //normalize
        for (i = 0; i < vahy.length; i++) {
            vahy[i] /= sumaNorm;
        }
       
        aktualizujVzdalenostPrazdnehoDokumentu();
        //return count of negatives
        return cnt;
    }

    /**
     * Actualize the distance of documents inside the cluster.
     */
    public void aktualizujVzdalenostiDokumentu() {
        //    aktualizujVzdalenostPrazdnehoDokumentu();
        for (DokumentShluku ds : dokumentyShluku) {
            ds.spoctiVzdalenostiDokumentu();
        }

    }

    /**
     * Actualize the distance of an empty document.
     */
    public void aktualizujVzdalenostPrazdnehoDokumentu() {
        vzdalenostPrazdnehoDokumentu = 0;
        double vahaTermu = Algoritmus.getVahaTermu();

        for (int i = 0; i < vahy.length; i++) {
            vzdalenostPrazdnehoDokumentu += vahaTermu * vahy[i];
        }
    }

    /**
     * Gets all keywords having the weight higher than zero. 
     * @return sorted list of keywords
     */
    public List<Slovo> getKlicovaSlova() {
        Slovo slovo;
        int id, i = 0;
        List<Slovo> a = new ArrayList<Slovo>();
        for (SlovoSuma s : data.getSlovaDokumenty().values()) {
            id = s.getId();
            if (getVahy()[id] > 0) {
                slovo = new Slovo(s.getJmeno());
                slovo.setId(id);
                slovo.setVaha(getVahy()[id]);
                a.add(slovo);
            }
        }
        //sorting
        Collections.sort(a, new Porovnej());
        //reverse - the highest must be first
        Collections.reverse(a);
        return a;
    }

    /**
     * Class for comparison of two terms based on their weight values.
     */
    private class Porovnej implements java.util.Comparator<Slovo> {

        @Override
        public int compare(Slovo o1, Slovo o2) {
            Double sl1 = o1.getVaha();
            Double sl2 = o2.getVaha();

            return sl1.compareTo(sl2);
        }
    }
}

