/* Text clustering method, SKWIC algorithm implementation
 * Model class used to validate the clustering results
 * Authors: Roman Milos, Vladimir Bartik, May 2011
 */
package dip.model.algoritmus;

import dip.model.Data;
import java.util.HashSet;

/**
 * Model class used to validate the clustering results
 * @author Roman
 */
public class Validace {

    //Arrays and variables necessary for computation and results presentation
    private Algoritmus alg;
    private int[][] tabulka;
    private int[] sumaTrid;
    private int[] maximalniTridaShluku;
    private int[] shlukyID;
    private double[] uspesnost;
    private String[] popisky;
    private double fSkore;
    private double prumer;
    private HashSet<Integer> prirazeneShluky;
    private HashSet<Integer> prirazeneTridy;
    private Data data = Data.getInstance();

    /**
     * Gets the actual average value of successfulness metric in the cluster. 
     * @return value of successfulness metric
     */
    public double getPrumer() {
        return prumer;
    }
    
    /**
     * Gets an actual value of F-Score metric. 
     * @return value of F-Score
     */
    public double getfSkore() {
        return fSkore;
    }
    
    /**
     * Gets an array of cluster identifiers. 
     * @return array of cluster identifiers
     */
    public int[] getShlukyID() {
        return shlukyID;
    }
    
    /**
     * Gets an array of document counts in all clusters. 
     * @return array of document counts
     */
    public int[] getSuma() {
        return sumaTrid;
    }
    
    /**
     * Gets the overall successfulness of clustering process for each cluster. 
     * @return array of F-Scores 
     */
    public double[] getUspesnost() {
        return uspesnost;
    }

    /**
     * Gets an array of category names assigned to clusters. 
     * @return array of category names
     */
    public String[] getPopisky() {
        return popisky;
    }

    /**
     * Gets the contingention table as 2D array. 
     * @return contingention table
     */
    public int[][] getTabulka() {
        return tabulka;
    }
    
    /**
     * Constructor, which executes the initialization of the table. 
     */
    public Validace(Algoritmus alg) {
        this.alg = alg;
        initTabulka();

    }

    /**
     * initialization of contingention table. Creates table labels and 
     * initializes all variables.
     */
    private void initTabulka() {

        popisky = new String[data.getKategorie().size() + 2];
        popisky[0] = "Cluster";
        int i = 1;
        for (String test : data.getKategorie().keySet()) {

            popisky[i] = test;
            data.getKategorie().put(test, i - 1);
            i++;
        }

        popisky[i] = "Not categorized";

        int pocetShluku = alg.getPocetShluku();
        tabulka = new int[pocetShluku][popisky.length - 1];

        maximalniTridaShluku = new int[pocetShluku];
        shlukyID = new int[popisky.length - 1];
        prirazeneShluky = new HashSet<Integer>(pocetShluku);
        prirazeneTridy = new HashSet<Integer>(shlukyID.length);
        uspesnost = new double[shlukyID.length];
        sumaTrid = new int[shlukyID.length];
    }

    /**
     * Cleans all tables.
     */
    private void vycistiTabulku() {
        int i, j;
        for (i = 0; i < tabulka.length; i++) {

            for (j = 0; j < tabulka[i].length; j++) {
                tabulka[i][j] = 0;
            }
        }
        for (j = 0; j < sumaTrid.length; j++) {
            sumaTrid[j] = 0;
        }

        for (j = 0; j < shlukyID.length; j++) {
            shlukyID[j] = -1;
        }
        for (j = 0; j < uspesnost.length; j++) {
            uspesnost[j] = 0;
        }
        prirazeneShluky.clear();
        prirazeneTridy.clear();
        fSkore = 0;
        prumer =0;
    }

    /**
     * Assigns a category ID for each cluster and counts the F-score metrics
     * and ratio of correctly assigned documents.
     */
    private void odhadniShluky() {
        int i, max, maxID;
        double tmp, max2;
        int pocetShluku = alg.getPocetShluku();
        
        if (sumaTrid[sumaTrid.length - 1] != 0) {
            return;
        }
        if (sumaTrid.length > pocetShluku + 1) {
          //  return;
        }

        // prvni kolo vyber prirad shluky pouze pokud je hodnota dominantni ve sloupci
        //i radku, skrtej obsazene kategorie a shluky
        for (int j = 0; j < tabulka[0].length-1; j++) {
            max = 0;
            maxID = -1;
            for (i = 0; i < tabulka.length ; i++) {
                if (!prirazeneShluky.contains(i)) {
                    if (max < tabulka[i][j]) {
                        max = tabulka[i][j];
                        maxID = i;
                    }
                }
            }
            if (maxID > -1) {
                if (tabulka[maxID][j] == maximalniTridaShluku[maxID]) {
                    prirazeneShluky.add(maxID);
                    prirazeneTridy.add(j);
                    shlukyID[j] = maxID;
                    uspesnost[j] = spoctiFSkore(maxID, j);
                    prumer+=tabulka[maxID][j];
                }
            }
        }
        //druhe kolo: pro vsechny sbivajici prvky kontingencni tabulky spocti
        //fskore a vybirej maxima, skrtej obsazene shluky a kategorie
        for (int j = 0; j < tabulka[0].length-1; j++) {
            if (prirazeneTridy.contains(j)) {
                continue;
            }
            max2 = 0;
            maxID = -1;
            for (i = 0; i < tabulka.length; i++) {
                if (!prirazeneShluky.contains(i)) {

                    if (0 < tabulka[i][j]) {
                        tmp = spoctiFSkore(i, j);
                        if (tmp > max2) {
                            maxID = i;
                            max2 = tmp;
                        }
                    }
                }
            }
            if (maxID > -1) {
                prirazeneShluky.add(maxID);
                prirazeneTridy.add(j);
                shlukyID[j] = maxID;
                uspesnost[j] = max2;
                prumer+=tabulka[maxID][j];
            }
        }
        fSkore = 0;
        for (i = 0; i < uspesnost.length; i++) {
            fSkore += uspesnost[i];
        }
        fSkore /= pocetShluku;
        prumer /=Data.getInstance().getDokumenty().size();
    }


    /**
     * Computes an F-score value at a given index in the contingention table. 
     * @param i first index
     * @param j second index
     * @return a value of F-score
     */
    private double spoctiFSkore(int i, int j) {
        double presnost = ((double) tabulka[i][j]) / alg.getShluky().get(i).getDokumentyShluku().size();
        double uplnost = ((double) tabulka[i][j]) / sumaTrid[j];
        return 2 * presnost * uplnost / (presnost + uplnost);
    }

    /**
     * Fills the contingention table with data from actual clusters.
     */
    public void naplnTabulku() {
        vycistiTabulku();
        String kategorie;
        Integer kat;
        int i = 0, j, konecI;
        for (Shluk shluk : alg.getShluky()) {
            for (DokumentShluku ds : shluk.getDokumentyShluku()) {
                kategorie = ds.getDokument().getKategorie();
                kat = data.getKategorie().get(kategorie);
                if (kat != null) {
                    tabulka[i][kat]++;
                } else {
                    tabulka[i][tabulka[i].length - 1]++;

                }
            }
            i++;
        }
        int max;
        for (i = 0; i < tabulka.length; i++) {
            konecI = tabulka.length - 1;
            max = 0;
            for (j = 0; j < tabulka[i].length; j++) {
                if (max < tabulka[i][j]) {
                    max = tabulka[i][j];
                }
                sumaTrid[j] += tabulka[i][j];
            }
            maximalniTridaShluku[i] = max;
        }
        odhadniShluky();
    }
}
